@@ -1,6 +1,6 @@
 {
 package Mite::App::Command;
-our $USES_MITE = q[Mite::Class];
+our $USES_MITE = "Mite::Class";
 use strict;
 use warnings;
 
@@ -9,7 +9,7 @@ BEGIN {
 
     use mro 'c3';
     our @ISA;
-    push @ISA, q[App::Cmd::Command];
+    push @ISA, "App::Cmd::Command";
 }
 
 sub new {
@@ -31,10 +31,6 @@ sub new {
     return $self;
 }
 
-defined ${^GLOBAL_PHASE}
-    or eval { require Devel::GlobalDestruction; 1 }
-    or do   { *Devel::GlobalDestruction::in_global_destruction = sub { undef; } };
-
 sub DESTROY {
     my $self  = shift;
     my $class = ref( $self ) || $self;
@@ -56,7 +52,6 @@ sub DESTROY {
 
 sub __META__ {
     no strict 'refs';
-    require mro;
     my $class      = shift; $class = ref($class) || $class;
     my $linear_isa = mro::get_linear_isa( $class );
     return {
