@@ -6,7 +6,7 @@ package Mite::Role;
 use Mite::Miteception;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.003001';
+our $VERSION   = '0.004000';
 
 use Path::Tiny;
 use B ();
@@ -36,6 +36,11 @@ has roles =>
   isa           => ArrayRef,
   builder       => sub { [] };
 
+has constants =>
+  is            => ro,
+  isa           => HashRef,
+  builder       => sub { {} };
+
 ##-
 
 sub skip_compiling {
@@ -231,6 +236,7 @@ sub compilation_stages {
         _compile_package
         _compile_uses_mite
         _compile_pragmas
+        _compile_constants
         _compile_with
         _compile_does
         _compile_composed_methods
@@ -309,6 +315,21 @@ sub _compile_composed_methods {
     return $code;
 }
 
+sub _compile_constants {
+    my $self = shift;
+    my %const = %{ $self->constants }
+        or return;
+
+    return join "\n",
+        'BEGIN {',
+        map(
+            sprintf( '    *%s = \&%s;',  $_, $const{$_} ),
+            sort keys %const
+        ),
+        '};',
+        '';
+}
+
 sub _compile_package {
     my $self = shift;
 
