@@ -24,10 +24,104 @@ or do {
     *Devel::GlobalDestruction::in_global_destruction = sub { undef; };
 };
 
+# Constants
+sub true  () { !!1 }
+sub false () { !!0 }
+sub ro    () { 'ro' }
+sub rw    () { 'rw' }
+sub rwp   () { 'rwp' }
+sub lazy  () { 'lazy' }
+sub bare  () { 'bare' }
+
+my $parse_mm_args = sub {
+    my $coderef = pop;
+    my $names   = [ map { ref($_) ? @$_ : $_ } @_ ];
+    ( $names, $coderef );
+};
+
 sub _is_compiling {
     return $ENV{MITE_COMPILE} ? 1 : 0;
 }
 
+sub import {
+    my $class = shift;
+    my %arg = map { lc($_) => 1 } @_;
+    my ( $caller, $file ) = caller;
+
+    # Turn on warnings and strict in the caller
+    warnings->import;
+    strict->import;
+
+    my $kind = $arg{'-role'} ? 'role' : 'class';
+
+    if( _is_compiling() ) {
+        require Mite::Project;
+        Mite::Project->default->inject_mite_functions(
+            package     => $caller,
+            file        => $file,
+            arg         => \%arg,
+            kind        => $kind,
+            shim        => $class,
+        );
+    }
+    else {
+        # Work around Test::Compile's tendency to 'use' modules.
+        # Mite.pm won't stand for that.
+        return if $ENV{TEST_COMPILE};
+
+        # Changes to this filename must be coordinated with Mite::Compiled
+        my $mite_file = $file . ".mite.pm";
+        if( !-e $mite_file ) {
+            require Carp;
+            Carp::croak("Compiled Mite file ($mite_file) for $file is missing");
+        }
+
+        {
+            local @INC = ('.', @INC);
+            require $mite_file;
+        }
+
+        $class->_inject_mite_functions( $caller, $file, $kind, \%arg );
+    }
+}
+
+sub _inject_mite_functions {
+    my ( $class, $caller, $file, $kind, $arg ) = ( shift, @_ );
+    my $requested = sub { $arg->{$_[0]} ? 1 : $arg->{'!'.$_[0]} ? 0 : $_[1]; };
+
+    no strict 'refs';
+    my $has = $class->_make_has( $caller, $file, $kind );
+    *{"$caller\::has"}   = $has if $requested->( has   => 1 );
+    *{"$caller\::param"} = $has if $requested->( param => 0 );
+    *{"$caller\::field"} = $has if $requested->( field => 0 );
+
+    *{ $caller .'::with' } = $class->_make_with( $caller, $file, $kind )
+        if $requested->( with => 1 );
+
+    *{ $caller .'::extends'} = sub {}
+        if $kind eq 'class' && $requested->( extends => 1 );
+
+    my $MM = ( $kind eq 'role' ) ? \@{"$caller\::METHOD_MODIFIERS"} : [];
+
+    for my $modifier ( qw/ before after around / ) {
+        next unless $requested->( $modifier => 1 );
+
+        if ( $kind eq 'class' ) {
+            *{"$caller\::$modifier"} = sub {
+                $class->$modifier( $caller, @_ );
+                return;
+            };
+        }
+        else {
+            *{"$caller\::$modifier"} = sub {
+                my ( $names, $coderef ) = &$parse_mm_args;
+                push @$MM, [ $modifier, $names, $coderef ];
+                return;
+            };
+        }
+    }
+}
+
 sub _make_has {
     my ( $class, $caller, $file, $kind ) = @_;
 
@@ -63,24 +157,13 @@ sub _make_has {
 
 sub _make_with {
     my ( $class, $caller, $file, $kind ) = @_;
-    my ( $cb_before, $cb_after );
 
     return sub {
         while ( @_ ) {
             my $role = shift;
             my $args = ref($_[0]) ? shift : undef;
             if ( $INC{'Role/Tiny.pm'} and 'Role::Tiny'->is_role( $role ) ) {
-                if ( $INC{'Role/Hooks.pm'} ) {
-                    $cb_before ||= \%Role::Hooks::CALLBACKS_BEFORE_APPLY;
-                    $cb_after  ||= \%Role::Hooks::CALLBACKS_AFTER_APPLY;
-                }
-                if ( $cb_before ) {
-                    $_->( $role, $caller ) for @{ $cb_before->{$role} || [] };
-                }
                 $class->_finalize_application_roletiny( $role, $caller, $args );
-                if ( $cb_after ) {
-                    $_->( $role, $caller ) for @{ $cb_after->{$role} || [] };
-                }
             }
             else {
                 $role->__FINALIZE_APPLICATION__( $caller, $args );
@@ -90,64 +173,34 @@ sub _make_with {
     };
 }
 
-sub _finalize_application_roletiny {
-    my ( $class, $role, $caller, $args ) = @_;
-    my $info = $Role::Tiny::INFO{$role};
-    for ( @{ $info->{modifiers} || [] } ) {
-        my @args = @$_;
-        my $kind = shift @args;
-        $class->$kind( $caller, @args );
-    }
-    return;
-}
+{
+    my ( $cb_before, $cb_after );
+    sub _finalize_application_roletiny {
+        my ( $class, $role, $caller, $args ) = @_;
 
-sub import {
-    my ( $class, $kind ) = @_;
-    my ( $caller, $file ) = caller;
-
-    # Turn on warnings and strict in the caller
-    warnings->import;
-    strict->import;
-
-    $kind ||= 'class';
-    $kind = ( $kind =~ /role/i ) ? 'role' : 'class';
-
-    if( _is_compiling() ) {
-        require Mite::Project;
-        my $method = "inject_mite_$kind\_functions";
-        Mite::Project->default->$method(
-            package     => $caller,
-            file        => $file,
-        );
-    }
-    else {
-        # Work around Test::Compile's tendency to 'use' modules.
-        # Mite.pm won't stand for that.
-        return if $ENV{TEST_COMPILE};
-
-        # Changes to this filename must be coordinated with Mite::Compiled
-        my $mite_file = $file . ".mite.pm";
-        if( !-e $mite_file ) {
-            require Carp;
-            Carp::croak("Compiled Mite file ($mite_file) for $file is missing");
+        if ( $INC{'Role/Hooks.pm'} ) {
+            $cb_before ||= \%Role::Hooks::CALLBACKS_BEFORE_APPLY;
+            $cb_after  ||= \%Role::Hooks::CALLBACKS_AFTER_APPLY;
+        }
+        if ( $cb_before ) {
+            $_->( $role, $caller ) for @{ $cb_before->{$role} || [] };
         }
 
-        {
-            local @INC = ('.', @INC);
-            require $mite_file;
+        my $info = $Role::Tiny::INFO{$role};
+        for ( @{ $info->{modifiers} || [] } ) {
+            my @args = @$_;
+            my $kind = shift @args;
+            $class->$kind( $caller, @args );
         }
 
-        my $method = "_inject_mite_$kind\_functions";
-        $class->$method( $caller, $file );
+        if ( $cb_after ) {
+            $_->( $role, $caller ) for @{ $cb_after->{$role} || [] };
+        }
+
+        return;
     }
 }
 
-my $parse_mm_args = sub {
-    my $coderef = pop;
-    my $names   = [ map { ref($_) ? @$_ : $_ } @_ ];
-    ( $names, $coderef );
-};
-
 {
     my $get_orig = sub {
         my ( $caller, $name ) = @_;
@@ -227,39 +280,6 @@ AROUND
     }
 }
 
-
-sub _inject_mite_class_functions {
-    my ( $class, $caller, $file ) = ( shift, @_ );
-
-    no strict 'refs';
-    *{ $caller .'::has' } = $class->_make_has( $caller, $file, 'class' );
-    *{ $caller .'::with' } = $class->_make_with( $caller, $file, 'class' );
-    *{ $caller .'::extends'} = sub {};
-
-    for my $mm ( qw/ before after around / ) {
-        *{"$caller\::$mm"} = sub {
-            $class->$mm( $caller, @_ );
-            return;
-        };
-    }
-}
-
-sub _inject_mite_role_functions {
-    my ( $class, $caller, $file ) = ( shift, @_ );
-
-    no strict 'refs';
-    *{ $caller .'::has' } = $class->_make_has( $caller, $file, 'role' );
-    *{ $caller .'::with' } = $class->_make_with( $caller, $file, 'role' );
-
-    my $MM = \@{"$caller\::METHOD_MODIFIERS"};
-    for my $modifier ( qw/ before after around / ) {
-        *{ $caller .'::'. $modifier } = sub {
-            my ( $names, $coderef ) = &$parse_mm_args;
-            push @$MM, [ $modifier, $names, $coderef ];
-        };
-    }
-}
-
 1;
 
 __END__
